#!/usr/bin/env python3


# ===========================================================================================
# Einbinden der Bibliotheken (Module)
# ===========================================================================================

# Allgemeines Modul für nummerische Berechnungen, siehe https://numpy.org
import numpy as np

# Modul, das zur Herstellung von Grafiken verwendet wird, siehe https://matplotlib.org
from matplotlib import pyplot as plt

# Die Bibliothek pandas (von PANel DAta) ist darauf spezialisiert Paneldaten zu verarbeiten,
# also zweidimensionale Tabellen (häufig Zeitreihen).
# In diesem Programm wird von dieser Bibliothek aber nur der CSV-Reader verwendet 
# siehe: https://pandas.pydata.org
import pandas as pd

# Die Bibliothek collections vereinfacht den Umgang mit Mengen und ähnlichen Ansammlungen
# Wir benötigen hier die Subklasse Counter, um das Zählen von Programmiersprachen nicht selbst 
# programmieren zu müssen (was aber auch nicht besonders schwer wäre)
# siehe https://docs.python.org/3/library/collections.html
from collections import Counter


# ===========================================================================================
# Daten einlesen und aufbereiten
# ===========================================================================================

# Wo stehen die Daten?
datasource = 'Daten/languages.csv'

# Immer wichtig zu wissen: Was steht das in den Daten?
#
# Es wurden Programmierer befragt, mit welchen Sprachen sie schon gearbeitete haben
# 87570 Programmierer haben gültige Antworten abgegeben, die ersten Zeilen der Datei sehen so aus:
#
# Responder_id,LanguagesWorkedWith
# 1,HTML/CSS;Java;JavaScript;Python
# 2,C++;HTML/CSS;Python
# 3,HTML/CSS
# 4,C;C++;C#;Python;SQL
# 5,C++;HTML/CSS;Java;JavaScript;Python;SQL;VBA

# Einlesen mit dem CSV-Reader von Pandas
data = pd.read_csv(datasource)

# Die Responder_id interessiert uns nicht, wir haben nur Interesse an der Spalte mit der Überschrift "LanguagesWorkedWith"
all_responds = data['LanguagesWorkedWith']

# Einen Zähler für die Sprachen erzeugen
language_counter = Counter()

# die csv-Datei Zeile für Zeile abarbeiten
for respond in all_responds:
	# jede gegebene Antwort in die einzelnen Sprachen zerlegen und diese Liste einfach in den Zähler schmeißen 
	language_counter.update(respond.split(";"))

# Die Top 15 Sprachen aus dem Zähler herausholen
# Dazu zuerst zwei leere Listen erzeugen 
languages  = []
popularity = []
# Dann die 15 häufigsten Sprachen in diese Listen einfügen
for item in language_counter.most_common(15):
	languages.append(item[0])
	popularity.append(item[1])

# Reihenfolge umdrehen (die Y-Achse geht von unten nach oben, wir wollen die Sprache auf Platz 1 aber nicht ganz unten stehen haben)
languages.reverse()
popularity.reverse()



# ===========================================================================================
# Erstellen des Balkendiagramms
# ===========================================================================================

### Style
# plt.style.use("ggplot")
plt.style.use("seaborn")
# plt.style.use("fivethirtyeight")

# ein Balkendiagramm ist ein horizonzales Säulendiagramm (BARs, Horizontal)
plt.barh(languages, popularity)

# Überschrift
plt.title("am häufigsten verwendete Programmiersprachen")

# Beschriftung der X-Achse
plt.xlabel("Anzahl")

# Variation des Layouts
plt.tight_layout()

# nur senkrechte Gitterlinien, keine waagrechten
plt.grid(axis = "x")

# Als Datei speichern
# plt.savefig("MatPlot.png")

# Als interaktive Grafik anzeigen
plt.show()
